/*
  Requires the following libraries:
  RF24 by TMRh20, Avamander
  MPU6050 by Electronic Cats

  Connections:
    Shown in the schematic
*/

#include <SPI.h>
#include "RF24.h"

// Radio Transmitter - NRF24L01+

RF24 radio(7, 8); // using pin 7 for the CE pin, and pin 8 for the CSN pin
uint8_t rocketAddress[] = "ROCKT";

const int payloadSize = 12; // *

union{
  struct {
    int16_t acc[3];
    int16_t gyr[3];
  } raw;
  char bytes[payloadSize];
} data;

// Sensor - Accelerometer and Gyroscope - MPU6050

#include "I2Cdev.h"
#include "MPU6050.h"
#include "Wire.h"

MPU6050 accelgyro;

#define THRESHOLD 1000

// Actuator - Servomotor

#include <Servo.h>

Servo servo;

#define START_ANGLE 110
#define END_ANGLE 30

void setup() {
  Serial.begin(115200);
  Serial.println("Started");

  // Configure Radio Transmitter - NRF24
  
  if (!radio.begin()) {
    Serial.println(F("Radio hardware not responding"));
    while (1) {}; // hold in infinite loop
  }

  if (radio.isChipConnected()){
    Serial.println("NRF24L01 connection successful");
  }
  else{
    Serial.println("NRF24L01 connection failed");
    while (1) {}; 
  }
  
  radio.setPALevel(RF24_PA_LOW);
  radio.setAutoAck(false);
  radio.setDataRate(RF24_250KBPS);
  
  // This node sends data
  radio.openWritingPipe(rocketAddress);

  // Configure Sensor - Accelerometer and Gyroscope - MPU6050
  Wire.begin();
  accelgyro.initialize();

  if(accelgyro.testConnection()){
    Serial.println("MPU6050 connection successful");
  }
  else{
    Serial.println("MPU6050 connection failed");
    while (1) {}; 
  }

  // Configure Actuator - Servo

  servo.attach(3);
  servo.write(START_ANGLE);

  // Wait for devices to be ready
  delay(500);
}

void loop() {
  // Get sensor data
  accelgyro.getMotion6(&data.raw.acc[0], &data.raw.acc[1], &data.raw.acc[2], &data.raw.gyr[0], &data.raw.gyr[1], &data.raw.gyr[2]);

  // Send sensor data
  radio.writeFast(data.bytes, payloadSize);

  // Detect if there's free fall and eject the parachute accordingly
  float mag = sqrt(sq((float)data.raw.acc[0]) + sq((float)data.raw.acc[1]) + sq((float)data.raw.acc[2]));

  if (mag < THRESHOLD){
    servo.write(END_ANGLE);
  }

  Serial.println("");
}
